/*
  Large integer division algorithm.

  04/2019 : F. Prill, DWD.

  The division algorithm is explained in

  P. B. Hansen: Multiple-Length Division Revisited: A Tour of the Minefield (192)
                Electrical Engineering and Computer Science Technical Reports. 166.
*/

#ifndef INT_DIV_H_
#define INT_DIV_H_

#include <stdint.h>

/* array representation has (w+1) digits: */
#define W_DIGITS 6

/* large integer division: int data type */
typedef int64_t t_lid_int;

/* data type for a natural number represented by an array of
   (W_DIGITS+1) digits in radix b. */
typedef t_lid_int t_large_int[W_DIGITS + 1];

#define Fail -1
#define iZero (t_lid_int) 0


void         print_large_int(const t_large_int x);
void         number_init(t_large_int *x, const t_lid_int val, const unsigned int m);
unsigned int length(const t_large_int x);
void         product(t_large_int *x, const t_large_int y, const t_lid_int k, const t_lid_int b, int* ret);
void         quotient(t_large_int *x, const t_large_int y, t_lid_int k, const t_lid_int b);
void         iremainder(t_large_int *x, const t_large_int y, t_lid_int k, const t_lid_int b);
t_lid_int    trial(const t_large_int r, const t_large_int d, const int k, const int m, const t_lid_int b, int* ret);
int          prefix_less_than(const t_large_int r, const t_large_int dq, const int k, const int m);
void         difference(t_large_int *r, const t_large_int dq, const int k, const int m, const t_lid_int b, int *ret);
int          division(const t_large_int x, const t_large_int y, t_large_int *q, t_large_int *r, const t_lid_int b);

void         convert_to_basis(const t_lid_int v_in, t_large_int *v_out, const int digits, const int istart);
t_lid_int    convert_to_decimal(const t_large_int v_in, const int istart, const int iend);

#endif /* INT_DIV_H_ */
